/*
    UHU-menu -- output plugin for XDG-compilant systems, e.g. KDE (3.2), Gnome (2.12)
    Copyright (C) 2003, 2005  Koblinger Egmont <egmont@uhulinux.hu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _GNU_SOURCE
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <errno.h>

#include "uhu-menu.h"

static void indent (FILE *f, int n)
{
	int i;
	for (i = 0; i < n; i++) fprintf(f, "\t");
}

static void dump (const struct item *item, const char *path, const char *path_tmp, FILE *ff, int n)
{
	int i;
	char *p;

	if (!item->issubmenu) return;
	p = strrchr(item->x_uhu_menu, '/');
	if (p == NULL) p = item->x_uhu_menu; else p++;
	if (p[0] == '\0') p = "UHU";
	indent(ff, n);
	fprintf(ff, "<Menu>\n");
	indent(ff, n+1);
	fprintf(ff, "<Name>");
	p = get_value(item, NULL, "Name", "");
	while (*p) {
		if ((*p >= '0' && *p <= '9') ||
		    (*p >= 'A' && *p <= 'Z') ||
		    (*p >= 'a' && *p <= 'z')) fputc(*p, ff);
		else fputc('_', ff);
		p++;
	}
	fprintf(ff, "</Name>\n");
	if (n == 0) {
		indent(ff, n+1);
		fprintf(ff, "<DefaultAppDirs/>\n");
		indent(ff, n+1);
		fprintf(ff, "<DefaultDirectoryDirs/>\n");
	}
	indent(ff, n+1);
	fprintf(ff, "<Directory>%s</Directory>\n", item->file_name);
	for (i = 0; i < item->subitems_nr; i++) {
		dump(&item->subitems[i], path, path_tmp, ff, n+1);
	}
	indent(ff, n+1);
	fprintf(ff, "<Include>\n");
	indent(ff, n+2);
	fprintf(ff, "<Category>X-UHU-Menu%s</Category>\n", item->x_uhu_menu);
	indent(ff, n+1);
	fprintf(ff, "</Include>\n");
	indent(ff, n);
	fprintf(ff, "</Menu>\n");
}

int write_menu (const struct item *item, const char *path)
{
	char path_tmp[PATH_MAX];
	char buf[PATH_MAX];
	FILE *ff;

	sprintf(path_tmp, "%s.tmp", path);
	rm_rf(path_tmp);
	mkdir(path_tmp, 0777);
	sprintf(buf, "%s/UHU.menu", path_tmp);
	ff = fopen(buf, "w");
	if (ff == NULL) {
		message_e(3, "fopen %s", buf);
	} else {
		fprintf(ff, "<!DOCTYPE Menu PUBLIC \"-//freedesktop//DTD Menu 1.0//EN\" \"http://www.freedesktop.org/standards/menu-spec/1.0/menu.dtd\">\n");
		if (item->subitems_nr > 0) dump(item, path, path_tmp, ff, 0);
		fclose(ff);
	}
	movetree(path_tmp, path);
	return 0;
}

int need_all_entries = 1;

